"""
RatTeil is a app to live stream the Holly Quran to youtube
Copyright (C) 2024  Ali Elfatih

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import random as r, os, requests
import subprocess, glob as g
# ratteil imports 
from ratteil.surahs import surah_list as su_list
import ratteil.dl as dl 
 
def surah_index_to_filename(x):
  """
    Convert number to surah filename.
  
    input: int; ex 1; filename will be
    output: str; 001.mp3
  """
  x = int(x)
  if x >= 100 and x <= 114:
    return  f"{x}.mp3"
  elif x < 10 and x > 0:
    return f"00{x}.mp3"
  elif x >= 10 and x < 100:
    return f"0{x}.mp3"
  else: 
    return f"{x} isn't valid surah index."
    
def filename_resolve(filename):
  """
    Get reciter code, surah number, and moshaf type.
    
    get reciter code surah number and moshaf type from surah filename to use them when generating stream title and description.
  """
  filename = filename[6:-4].split("@@")
  return [filename[-1],filename[0].split("/")[-1] ,filename[1]]

class SurahInfo:
  """
    Get information about specific surah
  """
  def __init__(self, index,code,mtype, recitersDB):
    self.id = index
    self.name = su_list[f"surah_{index}"]["name"]
    self.reciter = Reciter(code, recitersDB)
    self.moshaf = self.reciter.get_moshaf_name_url(int(mtype))
    self.dl_name = f"{code}@@{mtype}@@{self.id}.mp3"
    self.url = self.reciter.get_moshaf_name_url(int(mtype), True)

  def __str__(self):
    return f" سورة {self.name} برواية {self.moshaf} بصوت القاريء {self.reciter.name}."

class Reciter:
  """
    General information about reciter
    
    reciters available moshafs
    input: str; reciter code 
  """
  def __init__(self, reciter_code, recitersDB):
    self.code = reciter_code
    self.name = recitersDB[self.code]["name"]
    self.moshafs = recitersDB[self.code]["moshaf"]
    self.have_multi_moshafs = len(self.moshafs) > 1
    
    self.select_moshaf = r.choice(self.moshafs)
    self.selected_moshaf_type=self.select_moshaf["moshaf_type"]
    self.selected_moshaf_surahs_list = self.select_moshaf["surah_list"].split(",")
    self.selected_moshaf_url = self.select_moshaf["server"]
  
  def get_moshaf_name_url(self,mtype,url=False):
    for x in self.moshafs:
      if x["moshaf_type"] == mtype:
        if not url:
          return (x["name"])
        else:
          return (x["server"])
  
#made change test
class RatTeil:
  def __init__(self, recitersDB, recitersCodesDB, resourcesPath, ratteilPath):
    self.reciters_list = []
    self.reciters_with_multi_moshafs =  []
    self.stream_surahs_list = []
    self.streams = []
    self.recitersDB = recitersDB
    self.recitersCodesDB = recitersCodesDB
    self.resourcesPath = resourcesPath
    self.ratteilPath = ratteilPath
    self.stream_title = ""
    self.surah_index = None
    self.stream_description = ""
    self.selected_surahs_list = []
    
  def get_available_reciters_list(self):
    for code in self.recitersCodesDB:
      reciter = Reciter(code, self.recitersDB)
      self.reciters_list.append(f"code: {reciter.code}\tname: {reciter.name}")
    return self.reciters_list
      
  def get_reciters_with_multi_moshafs(self):
    for code in self.recitersCodesDB:
      reciter = Reciter(code, self.recitersDB)
      if reciter.have_multi_moshafs:
        self.reciters_with_multi_moshafs.append(code)
    return self.reciters_with_multi_moshafs
  
  def generate_random_reciters_list(self, list_length, reciters_codes_list):
    temp_list = []
    if list_length > len(reciters_codes_list):
      if len(reciters_codes_list) == 1:
        list_length = 1
      elif len(reciters_codes_list) < 10:
        list_length = r.randrange(1,len(reciters_codes_list)+1)
      else:
        list_length = r.randrange(3,6)
    while len(temp_list) != list_length:
      reciter_code = r.choice(reciters_codes_list)
      if reciter_code not in temp_list:
        temp_list.append(reciter_code)
    self.reciters_list = temp_list
    
    for x in temp_list: # fix repeatedly choose the same surah for a reciter
      reciter = Reciter(x, self.recitersDB)
      y = []
      for i in reciter.selected_moshaf_surahs_list:
        s = SurahInfo(i,x,reciter.selected_moshaf_type, self.recitersDB)
        self.selected_surahs_list.append(f"audios/{s.dl_name}")
        
  # need development
  def generate_random_surahs_list(self,list_length, surah_index):
    
    temp_list = []
    SHRANK = False
    if not list_length <= len(self.selected_surahs_list):
      list_length = len(self.selected_surahs_list)
    while len(temp_list) != list_length:
      if not surah_index:
        x = r.choice(self.selected_surahs_list)
      else:
        shrank_list = [] 
        for i in self.selected_surahs_list:
          if f"@@{surah_index}.mp3" in i:
            shrank_list.append(i)
        print(shrank_list)
        if len(shrank_list)==0:
          break
        if not SHRANK:
          list_length = len(shrank_list)
        x = r.choice(shrank_list)
      self.selected_surahs_list.remove(x)
      temp_list.append(x)
    self.stream_surahs_list = temp_list
    
  def get_length(self,file):
    """
      Get audio file duration.
    """
    try:
      
      fileLength = (float(subprocess.check_output(f'ffprobe -i {file} -show_entries format=duration -v quiet -of csv="p=0"', shell=True)[:-2])/60)
      fileLength = float(str(fileLength)[:6])
      return fileLength
    except subprocess.CalledProcessError:
      os.remove(file)
      return 0.0
  
  def prepare_stream_files(self,streamFiles):
    """
      Split a large stream into small streams.
      
      split stream to a number of streams of max length equal max_stream_length
      #default 75 minute.
    """
    streamLength = 0.0
    max_stream_length = 75.0 # 1:15 
    streams = []
    temp_stream = []
    
    while len(streamFiles) != 0:
      addfile = streamFiles[0]
      fileLength = self.get_length(addfile)
      if fileLength <= max_stream_length:
        if streamLength + fileLength <= max_stream_length :
          streamLength += fileLength
          if fileLength > 0.0:
            temp_stream.append(addfile)
          streamFiles.remove(addfile)
        else:
          streams.append(temp_stream)
          streamLength = 0.0 
          temp_stream = []
      else:
        print(f"skipping... {addfile}, see logfile for details.") # to avoid copyright claims while streaming; when using cc content without been added to cc holder allowed list
        f = open(f"{self.resourcesPath}/logs.txt", "a")
        f.write(f"Audio file with duration greater than {max_stream_length}m, will cuases copyrights issues when using cc content; {addfile} = {fileLength}m.\n")
        f.close()
        streamFiles.remove(addfile)
    streams.append(temp_stream)
    self.streams = streams
  
  def generate_stream_details(self, stream):
    """
      Generate title and description for stream
    
    """
    streamD = ""
    random_title = SurahInfo(*filename_resolve(r.choice(stream)), self.recitersDB).reciter.name 
    self.stream_title = "%s || القرآن الكريم بث مباشر" % random_title
    for surah in stream:
      info = SurahInfo(*filename_resolve(surah), self.recitersDB)
      streamD += f"\n{info}"
    
    self.stream_description = f"""{streamD}"""
  
  def stream(self, stream, sites):
    """
      Luanch streaming process
      
      call youtube api to create stream and start streaming stream content
    """
    def streamFixer(fixer,URI): # avoid repetition when streaming intro and end
      """
        Stream intro and conclusion video.
      """
      os.system(f"ffmpeg -loglevel error -re -i {fixer} -c:v libx264 -pix_fmt yuv420p -preset veryfast -b:v 800k -r 30.0 -g 60.0 -c:a aac -b:a 128k -ac 2 -maxrate 400k -bufsize 200k -shortest -strict experimental {URI}")
    
    fb_falg = False
    yt_falg = False
    yt_fb_falg = False
    if "youtube" in sites or "yt" in sites :
      yt_falg = True
    
    # fb stream setup 
    if "facebook" in sites or "fb" in sites:
      from ratteil.keys import RATTEIL_AC,RATTEIL_SE
      fb_falg = True
      params = {
      'status': 'LIVE_NOW',
      'access_token': RATTEIL_AC,
      'appsecret_proof': RATTEIL_SE,
      'title': self.stream_title,
      'description': self.stream_description,
      }
      response = requests.post('https://graph.facebook.com/v20.0/me/live_videos', params=params)
      fbr = response.content.decode()
      fbresponse = eval(fbr)
      urly = fbresponse["secure_stream_url"]
      fburl = ""
      urly = list(urly)
      for i in urly:
        if i != "\\":
          fburl = fburl+i
          
    # new  youtube stream event
    if yt_falg:
      os.system(f"python3 {self.ratteilPath}/create_stream.py -rPath {self.ratteilPath} --title '{self.stream_title}' --noauth_local_webserver --description '''{self.stream_description}''' -resPath {self.resourcesPath}")
    # getStream key
    YT_URL = None
    if len(sites) > 1:
      yt_fb_falg = True
      YOUTUBE_STREAM_KEY = open(f"{self.resourcesPath}/stream_key.txt").read().strip()
      FACEBOOk_STREAM_URL = fburl
      YT_URL = f"-f flv rtmp://a.rtmp.youtube.com/live2/{YOUTUBE_STREAM_KEY}"
      
      URI = f"-flags:v +global_header -f tee -map 0:v -map 1:a '[f=flv]rtmp://a.rtmp.youtube.com/live2/{YOUTUBE_STREAM_KEY}|[f=flv]{FACEBOOk_STREAM_URL}'"
      
    else:
      URI = "-f flv '''%s'''"
      if sites[0] == "facebook" or sites[0] == "fb":
        URI = URI % fburl #fbresponse["secure_stream_url"]
      else:
        YOUTUBE_STREAM_KEY = open(f"{self.resourcesPath}/stream_key.txt").read()
        URI = URI % f"rtmp://a.rtmp.youtube.com/live2/{YOUTUBE_STREAM_KEY}"
    # to avoid you cutting seconds from stream
    startingFile = f"{self.resourcesPath}/fixer/introduction.mp4"
    endingFile = f"{self.resourcesPath}/fixer/conclusion.mp4"
    if not fb_falg:
      streamFixer(startingFile,URI) #stream intro
      for file in stream: #stream a stream content
        imglist = g.glob(f"{self.resourcesPath}/imgs/*") 
        os.system(f"ffmpeg -loglevel error -re -loop 1 -f image2 -i {r.choice(imglist)} -i {file} -c:v libx264 -pix_fmt yuv420p -preset veryfast -b:v 800k -r 30.0 -g 60.0 -c:a aac -b:a 128k -ac 2 -maxrate 800k -bufsize 400k -shortest -strict experimental {URI}")
      streamFixer(endingFile,URI) #stream ending
    else:
      def merge_audios(audios, outPath):
        command = 'ffmpeg -loglevel error %s -filter_complex "%sconcat=n=%s:v=0:a=1" %s/fb_stream/merged.mp3'
        
        audio_cod = ""
        audio_inputs = ""
        n = len(audios)
        track_input = 0
        for audio in audios:
          audio_inputs = audio_inputs+" -i %s" % audio
          audio_cod = audio_cod+"[%s:a]" % track_input
          track_input+=1
        command = command % (audio_inputs, audio_cod,n,outPath)
        try:
          os.remove(f"{outPath}/fb_stream/merged.mp3")
        except:
          pass

        os.system(command)
      merge_audios(stream, self.resourcesPath)
      if yt_fb_falg:
        streamFixer(startingFile, YT_URL)
      imglist = g.glob(f"{self.resourcesPath}/imgs/*") 
      stream_file = f"{self.resourcesPath}/fb_stream/merged.mp3"
      if fb_falg and not yt_fb_falg:
        pass
      os.system(f"ffmpeg -loglevel error -re -loop 1 -f image2 -i {r.choice(imglist)} -i {stream_file} -c:v libx264 -pix_fmt yuv420p -preset veryfast -b:v 800k -r 30.0 -g 60.0 -c:a aac -b:a 128k -ac 2 -maxrate 800k -bufsize 400k -shortest -strict experimental {URI}")
      if yt_fb_falg:
        streamFixer(endingFile, YT_URL)
      
    if fb_falg:
      end_params = {
        'end_live_video': True,
        'access_token': RATTEIL_AC,
        'appsecret_proof': RATTEIL_SE
      }
      requests.post(f"https://graph.facebook.com/v20.0/{fbresponse['id']}", params=end_params)
      os.remove(f"{self.resourcesPath}/fb_stream/merged.mp3")
    
def stream_validation(ratteil,max_len, streamFiles, minimum, reciters_code, prevoius_len):
  """
    Assure audio files duration up to the minimum
    
    Check if audio files duration meet minimum requirement and if not; start
    revalidation process by downloading more audios until minimum requirement
    get met. 
  """
  #minimum by default 450.0 minutes #7.5 hours
  duration = 0.0
  if prevoius_len:
    duration = prevoius_len
  else:
    for file in streamFiles:
      file_len = ratteil.get_length(file)
      if file_len > 0.0:
        duration += file_len
      else:
        streamFiles.remove(file)
  
  if minimum > max_len:
    minimum = max_len 
  while duration < minimum: # download more audio files if downloaded file duration not met the minimum duration of stream
    if len(g.glob(f"{ratteil.resourcesPath}/audios/*")) != len(ratteil.reciters_list):
      ratteil.generate_random_reciters_list(1,reciters_code)
      ratteil.generate_random_surahs_list(1, ratteil.surah_index)
      x = ratteil.stream_surahs_list[0]
      surah = SurahInfo(*filename_resolve(x), ratteil.recitersDB).dl_name # downloading name for surah
      new_len,max_st = dl.dl(ratteil, max_len, ratteil.stream_surahs_list, ratteil.recitersDB, ratteil.resourcesPath, True, duration) # download selected surah
      duration = new_len
    else:
      minimum = duration
      break
    
  return duration



"""
RatTeil is a app to live stream the Holly Quran to youtube
Copyright (C) 2024  Ali Elfatih

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from . import ratteil 

if __name__ == "__main__":
  ratteil()


import os, ratteil.base as base
from pathlib import Path
def dl(streamer, max_len, surahs_list, recitersDB, resourcesPath, validation=False, previous_len=None):
  """
    Download audios files
  """
  MAX_ST = False
  current_len = 0 
  ensure_max = []
  if previous_len:
    current_len = previous_len
  
  DOWNLOAD_PATH = Path(f"{resourcesPath}/audios")
  if not validation: # avoid deleting audios on validation process 
    OLD_FILES = base.g.glob(f"{DOWNLOAD_PATH}/*")
    if len(OLD_FILES) >= 1:
      for file in OLD_FILES:
        y = file.split("/")[-1]
        z = "audios/%s" % y
        if z not in surahs_list:
          os.remove(file)
        else:
          surahs_list.remove(z)
          file_len = streamer.get_length(file)
          current_len += file_len
          ensure_max.append({
            "file": file,
            "file_len": file_len,
            "less10": file_len < 10
          })

  for x in surahs_list:
    surah = base.SurahInfo(*base.filename_resolve(x), recitersDB)
    SURAH_PATH = DOWNLOAD_PATH / surah.dl_name
    #os.system("clear")
    URL = f"{surah.url}{base.surah_index_to_filename(surah.id)}"
    if not SURAH_PATH.exists():
      if current_len < max_len:
        os.system(f"wget {URL}  -O {SURAH_PATH}")
        file_len = streamer.get_length(SURAH_PATH)
        current_len += file_len
        ensure_max.append({
          "file": SURAH_PATH,
          "file_len": file_len,
          "less10": file_len < 10
        })
      else:
        MAX_ST = True
  
  #current length to max_length reminder
  cur_mx_rem = abs(current_len - max_len)
  if cur_mx_rem > 1:
    for y in range(1,3):
      for i in ensure_max:
        xyz = abs(current_len - i["file_len"])
        if xyz >= (max_len+3) or (xyz >= max_len+1 and xyz <= (max_len+1)):
          current_len -= i["file_len"]
          os.remove(i["file"])
          ensure_max.remove(i)
    print("current : ", current_len) 
       
  return current_len, MAX_ST


#!/usr/bin/python


import httplib2
import os
import sys
import datetime
from apiclient.discovery import build
from apiclient.errors import HttpError
from oauth2client.client import flow_from_clientsecrets
from oauth2client.file import Storage
from oauth2client.tools import argparser, run_flow

# The CLIENT_SECRETS_FILE variable specifies the name of a file that contains
# the OAuth 2.0 information for this application, including its client_id and
# client_secret. You can acquire an OAuth 2.0 client ID and client secret from
# the {{ Google Cloud Console }} at
# {{ https://cloud.google.com/console }}.
# Please ensure that you have enabled the YouTube Data API for your project.
# For more information about using OAuth2 to access the YouTube Data API, see:
#   https://developers.google.com/youtube/v3/guides/authentication
# For more information about the client_secrets.json file format, see:
#   https://developers.google.com/api-client-library/python/guide/aaa_client_secrets
CLIENT_SECRETS_FILE = "%s/client_secrets.json"

# This OAuth 2.0 access scope allows for full read/write access to the
# authenticated user's account.
YOUTUBE_READ_WRITE_SCOPE = "https://www.googleapis.com/auth/youtube"
YOUTUBE_API_SERVICE_NAME = "youtube"
YOUTUBE_API_VERSION = "v3"

# This variable defines a message to display if the CLIENT_SECRETS_FILE is
# missing.
MISSING_CLIENT_SECRETS_MESSAGE = """
WARNING: Please configure OAuth 2.0

To make this sample run you will need to populate the client_secrets.json file
found at:

   %s

with information from the {{ Cloud Console }}
{{ https://cloud.google.com/console }}

For more information about the client_secrets.json file format, please visit:
https://developers.google.com/api-client-library/python/guide/aaa_client_secrets
""" % os.path.abspath(os.path.join(os.path.dirname(__file__),
                                   CLIENT_SECRETS_FILE))

def get_authenticated_service(args):
  flow = flow_from_clientsecrets(CLIENT_SECRETS_FILE,
    scope=YOUTUBE_READ_WRITE_SCOPE,
    message=MISSING_CLIENT_SECRETS_MESSAGE)

  storage = Storage("%s-oauth2.json" % sys.argv[0])
  credentials = storage.get()

  if credentials is None or credentials.invalid:
    credentials = run_flow(flow, storage, args)

  return build(YOUTUBE_API_SERVICE_NAME, YOUTUBE_API_VERSION,
    http=credentials.authorize(httplib2.Http()))

# Create a liveBroadcast resource and set its title, scheduled start time,
# scheduled end time, and privacy status.
def insert_broadcast(youtube, options):
  insert_broadcast_response = youtube.liveBroadcasts().insert(
    part="snippet,status,contentDetails",
    body=dict(
      snippet=dict(
        title=options.title,
        description=options.description,
        scheduledStartTime=options.start_time,
        scheduledEndTime=options.end_time
      ),
      status=dict(
        privacyStatus=options.privacy_status
      ),
      contentDetails=dict(
        enableAutoStart=True,
        enableAutoStop=True
      )
    )
  ).execute()

  snippet = insert_broadcast_response["snippet"]
  
  print("Broadcast '%s' with title '%s' was published at '%s'." % (
    insert_broadcast_response["id"], snippet["title"], snippet["publishedAt"]))
  return insert_broadcast_response["id"]

# Create a liveStream resource and set its title, format, and ingestion type.
# This resource describes the content that you are transmitting to YouTube.
def insert_stream(youtube, options):
  insert_stream_response = youtube.liveStreams().insert(
    part="snippet,cdn,contentDetails",
    body=dict(
      snippet=dict(
        title=options.title,
        description=options.description
      ),
      cdn=dict(
        #format="1080p",
        resolution="1080p",
        frameRate="30fps",
        ingestionType="rtmp"
      ),
      contentDetails=dict(
        isReusable=True,
        enableAutoStart=True,
        enableAutoStop=True
      )
    )
  ).execute()

  snippet = insert_stream_response["snippet"]
  
  stream_key = insert_stream_response["cdn"]["ingestionInfo"]["streamName"]
  
  f = open(f"{options.resourcesPath}/stream_key.txt","w")
  f.write(stream_key)
  f.close()
  print("Stream '%s' with title '%s' was inserted." % (
    insert_stream_response["id"], snippet["title"]))
  return insert_stream_response["id"]

# Bind the broadcast to the video stream. By doing so, you link the video that
# you will transmit to YouTube to the broadcast that the video is for.
def bind_broadcast(youtube, broadcast_id, stream_id):
  bind_broadcast_response = youtube.liveBroadcasts().bind(
    part="id,contentDetails",
    id=broadcast_id,
    streamId=stream_id
  ).execute()
  
 # print(bind_broadcast_response)
  print("Broadcast '%s' was bound to stream '%s'." % (
    bind_broadcast_response["id"],
    bind_broadcast_response["contentDetails"]["boundStreamId"]))

if __name__ == "__main__":
  dt = datetime.datetime.now() + datetime.timedelta(minutes=5)
  
  edt = dt + datetime.timedelta(hours=1,minutes=45)
  dt = dt.isoformat()
  edt = edt.isoformat()
  print(f"start after : {dt} \n end at: {edt}")
#  argparser.add_argument("--broadcast-title", help="Broadcast title",
#    default="Moddaker مُدّكر is live : تلاوة ")
  argparser.add_argument("--privacy-status", help="Broadcast privacy status",
    default="public")
  argparser.add_argument("--start-time", help="Scheduled start time",
    default=dt)
  argparser.add_argument("--end-time", help="Scheduled end time",
    default= edt)
  argparser.add_argument("-rPath", dest="ratteilPath", required=True)
  argparser.add_argument("-resPath", dest="resourcesPath", required=True)
  argparser.add_argument("--title", dest="title", required=True)
  argparser.add_argument("--description", dest="description", required=True)
#  argparser.add_argument("--stream-title", help="Stream title",
#    default="Moddaker مُدّكر is live : تلاوة")
#  argparser.add_argument("--stream-description", help="Stream description", default="Content in this video licensed under CC-BY 3.p URI: https://creativecommons.org/licenses/by/3.0/ \n ")
  args = argparser.parse_args()
  CLIENT_SECRETS_FILE = CLIENT_SECRETS_FILE % args.ratteilPath
  print(args.resourcesPath)
  youtube = get_authenticated_service(args)
  try:
    broadcast_id = insert_broadcast(youtube, args)
    stream_id = insert_stream(youtube, args)
    bind_broadcast(youtube, broadcast_id, stream_id)
  except HttpError as e:
    print("An HTTP error %d occurred:\n%s" % (e.resp.status, e.content))


import glob as g
def merge_audios(audios):
  command = 'ffmpeg %s -filter_complex "[0:a][1:a][2:a]concat=n=3:v=0:a=1" output.mp'
  
  audio_inputs = ""
  n = len(audios)
  track_input = 0
  for audio in audios:
    audio_inputs = audio_inputs+"[%s:a]" % track_input
    track_input+=1
  
  print(command % audio_inputs)
  

merge_audios(g.glob("*.mp3"))


"""
RatTeil is a app to live stream the Holly Quran to youtube
Copyright (C) 2024  Ali Elfatih

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
def ratteil():
  import os, time, ratteil.base as base, random as r, ratteil.dl as dl, glob as g
  import argparse
  from ratteil.base import RatTeil
  from pathlib import Path
  from ratteil.reciters import reciters_code as recitersCodesDB, reciters as recitersDB
  
  ####
  
  USER_HOME = Path.home()
  RATTEIL_RESOURCES= USER_HOME / "RatTeil-resources"
  RATTEIL_PATH = Path(__file__).parents[0]
  
  if not RATTEIL_RESOURCES.exists():
    #first time setup
    RATTEIL_RESOURCES.mkdir()
    os.mkdir(f"{RATTEIL_RESOURCES}/audios")
    os.mkdir(f"{RATTEIL_RESOURCES}/imgs")
    os.mkdir(f"{RATTEIL_RESOURCES}/fixer")
    os.mkdir(f"{RATTEIL_RESOURCES}/fb_stream")
    os.system(f"cp -r {RATTEIL_PATH}/fixer/* {RATTEIL_RESOURCES}/fixer/")
    os.system(f"cp -r {RATTEIL_PATH}/imgs/* {RATTEIL_RESOURCES}/imgs/")
  
  def PRE_LUNCHER(options, RATTEIL_RESOURCES):
    streamer = RatTeil(recitersDB, recitersCodesDB, RATTEIL_RESOURCES, RATTEIL_PATH)
    temp_reciters_list=[]
    streamer.surah_index = options.surah
    if options.reciter:
      options.reciters_number == len(options.reciter)
      temp_reciters_list = options.reciter 
    else:
      temp_reciters_list = recitersCodesDB
    
    if not options.no_download:
      streamer.generate_random_reciters_list(options.reciters_number, temp_reciters_list)
      #test and improve needed 
      streamer.generate_random_surahs_list(options.surahs_number, options.surah)
      
      previous_len, max_st = dl.dl(streamer, options.max_len, streamer.stream_surahs_list, streamer.recitersDB, streamer.resourcesPath)
      
      if not options.no_validation:
        minimum_stream_length = 0 
        if options.stream_numbers:
          minimum_stream_length = options.stream_numbers * 75.0
        elif options.surahs_number:
          minimum_stream_length = 0
        else:
          minimum_stream_length = options.minimum_stream_length 
          if not max_st:
            aea = base.stream_validation(streamer, options.max_len, g.glob(f"{streamer.resourcesPath}/audios/*"), minimum_stream_length,temp_reciters_list, previous_len)
    
    
    luncher(streamer, sites=options.sites)
  
  def luncher(streamer, sites):
    streamer.prepare_stream_files(g.glob(f"{streamer.resourcesPath}/audios/*"))
    
    for stream in streamer.streams:
      streamer.generate_stream_details(stream)
      streamer.stream(stream, sites)
      time.sleep(60)  


  parser = argparse.ArgumentParser(prog="RatTeil",
  description="Stream The Holly Quran to a youtube channel.", 
  epilog="""
    RatTeil  Copyright (C) 2024  Ali Elfatih. \n
  This program comes with ABSOLUTELY NO WARRANTY. \n
  This is free software, and you are welcome to redistribute it under certain conditions; \n \t type `ratteil -c' for details.
    Made with ♡ & ^_^ by Moddaker Team""",
  formatter_class=argparse.ArgumentDefaultsHelpFormatter
  )
  parser.add_argument("-c", "--copyright", help="Show Copyright LICENSE.",action="store_const",const=True, dest="copyright")
  parser.add_argument("-l","--list", metavar="", help="available reciters", action="store_const", const=True, default=False, dest="reciters_list")
  parser.add_argument("-mx", "--max", metavar="",help="max stream length in minutes", type=float, default=450, dest="max_len")
  parser.add_argument("-m","--min", metavar="", type=float, help="minimum streams length in minutes", default=120.0, dest="minimum_stream_length")
  parser.add_argument("-ms","--streams",metavar="", type=float, help="minimum streams number. Note that each stream'll be 75 minute or less", dest="stream_numbers")
  parser.add_argument("-n", metavar="", type=int, help="number of reciters per stream", default=r.randrange(3,7), dest="reciters_number")
  parser.add_argument("-R","--reciter", metavar="", help="choose reciter or reciters by code", dest="reciter", nargs="*", choices=recitersCodesDB)
  parser.add_argument("-r", "--resume",metavar="", help="resume previous stream", action="store_const", const=True, default=False, dest="resume_stream")
  parser.add_argument("-sn", metavar="", type=int, help="Surahs number per stream", default=r.randrange(20,50), dest="surahs_number")
  parser.add_argument("-s","--surah", metavar="", type=int,choices=[i for i in range(1,115)] ,help="choose surah by index", dest="surah")
  parser.add_argument("--no-download", metavar="", default=False, help="stream existing files; no new downloads", dest="no_download", action='store_const', const=True)
  parser.add_argument("--no-validation", metavar="", help="stop minimum streams validation", default=False, action="store_const", const=True, dest="no_validation")
  parser.add_argument("-t","--sites", metavar="", help="site to stream to", dest="sites", choices=["youtube","yt","fb","facebook"], nargs="*")
  parser.add_argument("-d", "--dev",dest="dev", action="store_const", const=True)
  

  args = parser.parse_args()
  if args.dev:
    from ratteil.base_test import recs as recitersDB
    
  if ((not args.sites) and (not args.reciters_list)) and (not args.copyright):
    os.system("clear")
    print("!! sites are required at least one site !! \n")
    print("ex:\n $ ratteil -t youtube \n")
    
    exit(0)
    
  if args.reciters_list:
    streamer = RatTeil(recitersDB,recitersCodesDB, RATTEIL_RESOURCES,RATTEIL_PATH)
    reciters_list = streamer.get_available_reciters_list()
    reciters_list.sort()
    for reciter in reciters_list:
      print(reciter)
      
    print("""
      RatTeil is a app to live stream the Holly Quran to youtube
      Copyright (C) 2024  Ali Elfatih
      
      This program is free software: you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation, either version 3 of the License, or any later version.
      
      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.
      
      You should have received a copy of the GNU General Public License
      along with this program.  If not, see <https://www.gnu.org/licenses/>.
      """)
  elif args.copyright:
    copyright_file = open(RATTEIL_PATH / "LICENSE", "r")
    for line in copyright_file:
      print(line)
    exit(0)
  else:
    PRE_LUNCHER(args, RATTEIL_RESOURCES)
